package org.antlr.misc;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import org.antlr.misc.IntSet;
import org.antlr.misc.Interval;
import org.antlr.misc.IntervalSet;
import org.antlr.tool.Grammar;

public class BitSet implements IntSet, Cloneable {
  protected static final int BITS = 64;
  
  protected static final int LOG_BITS = 6;
  
  protected static final int MOD_MASK = 63;
  
  protected long[] bits;
  
  public BitSet() {
    this(64);
  }
  
  public BitSet(long[] bits_) {
    this.bits = bits_;
  }
  
  public BitSet(int nbits) {
    this.bits = new long[(nbits - 1 >> 6) + 1];
  }
  
  public void add(int el) {
    int n = wordNumber(el);
    if (n >= this.bits.length)
      growToInclude(el); 
    this.bits[n] = this.bits[n] | bitMask(el);
  }
  
  public void addAll(IntSet set) {
    if (set instanceof org.antlr.misc.BitSet) {
      orInPlace((org.antlr.misc.BitSet)set);
    } else if (set instanceof IntervalSet) {
      IntervalSet other = (IntervalSet)set;
      for (Interval I : other.intervals)
        orInPlace(range(I.a, I.b)); 
    } else {
      throw new IllegalArgumentException("can't add " + set.getClass().getName() + " to BitSet");
    } 
  }
  
  public void addAll(int[] elements) {
    if (elements == null)
      return; 
    for (int i = 0; i < elements.length; i++) {
      int e = elements[i];
      add(e);
    } 
  }
  
  public void addAll(Iterable<Integer> elements) {
    if (elements == null)
      return; 
    for (Integer element : elements)
      add(element.intValue()); 
  }
  
  public IntSet and(IntSet a) {
    org.antlr.misc.BitSet s = (org.antlr.misc.BitSet)clone();
    s.andInPlace((org.antlr.misc.BitSet)a);
    return s;
  }
  
  public void andInPlace(org.antlr.misc.BitSet a) {
    int min = Math.min(this.bits.length, a.bits.length);
    int i;
    for (i = min - 1; i >= 0; i--)
      this.bits[i] = this.bits[i] & a.bits[i]; 
    for (i = min; i < this.bits.length; i++)
      this.bits[i] = 0L; 
  }
  
  private static long bitMask(int bitNumber) {
    int bitPosition = bitNumber & 0x3F;
    return 1L << bitPosition;
  }
  
  public void clear() {
    for (int i = this.bits.length - 1; i >= 0; i--)
      this.bits[i] = 0L; 
  }
  
  public void clear(int el) {
    int n = wordNumber(el);
    if (n >= this.bits.length)
      growToInclude(el); 
    this.bits[n] = this.bits[n] & (bitMask(el) ^ 0xFFFFFFFFFFFFFFFFL);
  }
  
  public Object clone() {
    org.antlr.misc.BitSet s;
    try {
      s = (org.antlr.misc.BitSet)super.clone();
      s.bits = new long[this.bits.length];
      System.arraycopy(this.bits, 0, s.bits, 0, this.bits.length);
    } catch (CloneNotSupportedException e) {
      throw new InternalError();
    } 
    return s;
  }
  
  public int size() {
    int deg = 0;
    for (int i = this.bits.length - 1; i >= 0; i--) {
      long word = this.bits[i];
      if (word != 0L)
        for (int bit = 63; bit >= 0; bit--) {
          if ((word & 1L << bit) != 0L)
            deg++; 
        }  
    } 
    return deg;
  }
  
  public boolean equals(Object other) {
    if (other == null || !(other instanceof org.antlr.misc.BitSet))
      return false; 
    org.antlr.misc.BitSet otherSet = (org.antlr.misc.BitSet)other;
    int n = Math.min(this.bits.length, otherSet.bits.length);
    int i;
    for (i = 0; i < n; i++) {
      if (this.bits[i] != otherSet.bits[i])
        return false; 
    } 
    if (this.bits.length > n) {
      for (i = n + 1; i < this.bits.length; i++) {
        if (this.bits[i] != 0L)
          return false; 
      } 
    } else if (otherSet.bits.length > n) {
      for (i = n + 1; i < otherSet.bits.length; i++) {
        if (otherSet.bits[i] != 0L)
          return false; 
      } 
    } 
    return true;
  }
  
  public void growToInclude(int bit) {
    int newSize = Math.max(this.bits.length << 1, numWordsToHold(bit));
    long[] newbits = new long[newSize];
    System.arraycopy(this.bits, 0, newbits, 0, this.bits.length);
    this.bits = newbits;
  }
  
  public boolean member(int el) {
    int n = wordNumber(el);
    if (n >= this.bits.length)
      return false; 
    return ((this.bits[n] & bitMask(el)) != 0L);
  }
  
  public int getSingleElement() {
    for (int i = 0; i < this.bits.length << 6; i++) {
      if (member(i))
        return i; 
    } 
    return -7;
  }
  
  public boolean isNil() {
    for (int i = this.bits.length - 1; i >= 0; i--) {
      if (this.bits[i] != 0L)
        return false; 
    } 
    return true;
  }
  
  public IntSet complement() {
    org.antlr.misc.BitSet s = (org.antlr.misc.BitSet)clone();
    s.notInPlace();
    return s;
  }
  
  public IntSet complement(IntSet set) {
    if (set == null)
      return complement(); 
    return set.subtract(this);
  }
  
  public void notInPlace() {
    for (int i = this.bits.length - 1; i >= 0; i--)
      this.bits[i] = this.bits[i] ^ 0xFFFFFFFFFFFFFFFFL; 
  }
  
  public void notInPlace(int maxBit) {
    notInPlace(0, maxBit);
  }
  
  public void notInPlace(int minBit, int maxBit) {
    growToInclude(maxBit);
    for (int i = minBit; i <= maxBit; i++) {
      int n = wordNumber(i);
      this.bits[n] = this.bits[n] ^ bitMask(i);
    } 
  }
  
  private int numWordsToHold(int el) {
    return (el >> 6) + 1;
  }
  
  public static org.antlr.misc.BitSet of(int el) {
    org.antlr.misc.BitSet s = new org.antlr.misc.BitSet(el + 1);
    s.add(el);
    return s;
  }
  
  public static org.antlr.misc.BitSet of(Collection<? extends Integer> elements) {
    org.antlr.misc.BitSet s = new org.antlr.misc.BitSet();
    for (Integer el : elements)
      s.add(el.intValue()); 
    return s;
  }
  
  public static org.antlr.misc.BitSet of(IntSet set) {
    if (set == null)
      return null; 
    if (set instanceof org.antlr.misc.BitSet)
      return (org.antlr.misc.BitSet)set; 
    if (set instanceof IntervalSet) {
      org.antlr.misc.BitSet s = new org.antlr.misc.BitSet();
      s.addAll(set);
      return s;
    } 
    throw new IllegalArgumentException("can't create BitSet from " + set.getClass().getName());
  }
  
  public static org.antlr.misc.BitSet of(Map<? extends Integer, ?> elements) {
    return of(elements.keySet());
  }
  
  public static org.antlr.misc.BitSet range(int a, int b) {
    org.antlr.misc.BitSet s = new org.antlr.misc.BitSet(b + 1);
    for (int i = a; i <= b; i++) {
      int n = wordNumber(i);
      s.bits[n] = s.bits[n] | bitMask(i);
    } 
    return s;
  }
  
  public IntSet or(IntSet a) {
    if (a == null)
      return this; 
    org.antlr.misc.BitSet s = (org.antlr.misc.BitSet)clone();
    s.orInPlace((org.antlr.misc.BitSet)a);
    return s;
  }
  
  public void orInPlace(org.antlr.misc.BitSet a) {
    if (a == null)
      return; 
    if (a.bits.length > this.bits.length)
      setSize(a.bits.length); 
    int min = Math.min(this.bits.length, a.bits.length);
    for (int i = min - 1; i >= 0; i--)
      this.bits[i] = this.bits[i] | a.bits[i]; 
  }
  
  public void remove(int el) {
    int n = wordNumber(el);
    if (n >= this.bits.length)
      growToInclude(el); 
    this.bits[n] = this.bits[n] & (bitMask(el) ^ 0xFFFFFFFFFFFFFFFFL);
  }
  
  private void setSize(int nwords) {
    long[] newbits = new long[nwords];
    int n = Math.min(nwords, this.bits.length);
    System.arraycopy(this.bits, 0, newbits, 0, n);
    this.bits = newbits;
  }
  
  public int numBits() {
    return this.bits.length << 6;
  }
  
  public int lengthInLongWords() {
    return this.bits.length;
  }
  
  public boolean subset(org.antlr.misc.BitSet a) {
    if (a == null)
      return false; 
    return and(a).equals(this);
  }
  
  public void subtractInPlace(org.antlr.misc.BitSet a) {
    if (a == null)
      return; 
    for (int i = 0; i < this.bits.length && i < a.bits.length; i++)
      this.bits[i] = this.bits[i] & (a.bits[i] ^ 0xFFFFFFFFFFFFFFFFL); 
  }
  
  public IntSet subtract(IntSet a) {
    if (a == null || !(a instanceof org.antlr.misc.BitSet))
      return null; 
    org.antlr.misc.BitSet s = (org.antlr.misc.BitSet)clone();
    s.subtractInPlace((org.antlr.misc.BitSet)a);
    return s;
  }
  
  public List<Integer> toList() {
    throw new NoSuchMethodError("BitSet.toList() unimplemented");
  }
  
  public int[] toArray() {
    int[] elems = new int[size()];
    int en = 0;
    for (int i = 0; i < this.bits.length << 6; i++) {
      if (member(i))
        elems[en++] = i; 
    } 
    return elems;
  }
  
  public long[] toPackedArray() {
    return this.bits;
  }
  
  public String toString() {
    return toString(null);
  }
  
  public String toString(Grammar g) {
    StringBuilder buf = new StringBuilder();
    String separator = ",";
    boolean havePrintedAnElement = false;
    buf.append('{');
    for (int i = 0; i < this.bits.length << 6; i++) {
      if (member(i)) {
        if (i > 0 && havePrintedAnElement)
          buf.append(separator); 
        if (g != null) {
          buf.append(g.getTokenDisplayName(i));
        } else {
          buf.append(i);
        } 
        havePrintedAnElement = true;
      } 
    } 
    buf.append('}');
    return buf.toString();
  }
  
  public String toString(String separator, List<String> vocabulary) {
    if (vocabulary == null)
      return toString(null); 
    String str = "";
    for (int i = 0; i < this.bits.length << 6; i++) {
      if (member(i)) {
        if (str.length() > 0)
          str = str + separator; 
        if (i >= vocabulary.size()) {
          str = str + "'" + (char)i + "'";
        } else if (vocabulary.get(i) == null) {
          str = str + "'" + (char)i + "'";
        } else {
          str = str + (String)vocabulary.get(i);
        } 
      } 
    } 
    return str;
  }
  
  public String toStringOfHalfWords() {
    StringBuilder s = new StringBuilder();
    for (int i = 0; i < this.bits.length; i++) {
      if (i != 0)
        s.append(", "); 
      long tmp = this.bits[i];
      tmp &= 0xFFFFFFFFL;
      s.append(tmp);
      s.append("UL");
      s.append(", ");
      tmp = this.bits[i] >>> 32L;
      tmp &= 0xFFFFFFFFL;
      s.append(tmp);
      s.append("UL");
    } 
    return s.toString();
  }
  
  public String toStringOfWords() {
    StringBuilder s = new StringBuilder();
    for (int i = 0; i < this.bits.length; i++) {
      if (i != 0)
        s.append(", "); 
      s.append(this.bits[i]);
      s.append("L");
    } 
    return s.toString();
  }
  
  public String toStringWithRanges() {
    return toString();
  }
  
  private static int wordNumber(int bit) {
    return bit >> 6;
  }
}
